@@ -1,9 +1,13 @@
+import contextlib
+import datetime
 import json
 import logging
 import re
-from typing import Any, Dict, List, Optional, Tuple
+import time
+from typing import Any, Dict, List, Optional, Tuple, Union
 from urllib import parse as urlparse
 
+import pytz
 from botocore.utils import InvalidArnException
 from jsonpatch import apply_patch
 from jsonpointer import JsonPointerException
@@ -14,6 +18,7 @@ from requests.models import Response
 from localstack import config
 from localstack.constants import (
     APPLICATION_JSON,
+    HEADER_LOCALSTACK_EDGE_URL,
     LOCALHOST_HOSTNAME,
     PATH_USER_REQUEST,
     TEST_AWS_ACCOUNT_ID,
@@ -24,9 +29,20 @@ from localstack.utils import common
 from localstack.utils.aws import aws_stack
 from localstack.utils.aws.aws_responses import requests_error_response_json, requests_response
 from localstack.utils.aws.aws_stack import parse_arn
+from localstack.utils.aws.request_context import MARKER_APIGW_REQUEST_REGION, THREAD_LOCAL
+from localstack.utils.strings import long_uid
 
 LOG = logging.getLogger(__name__)
 
+REQUEST_TIME_DATE_FORMAT = "%d/%b/%Y:%H:%M:%S %z"
+
+# regex path pattern for user requests, handles stages like $default
+PATH_REGEX_USER_REQUEST = (
+    r"^/restapis/([A-Za-z0-9_\\-]+)(?:/([A-Za-z0-9\_($|%%24)\\-]+))?/%s/(.*)$" % PATH_USER_REQUEST
+)
+# URL pattern for invocations
+HOST_REGEX_EXECUTE_API = r"(?:.*://)?([a-zA-Z0-9-]+)\.execute-api\.(localhost.localstack.cloud|([^\.]+)\.amazonaws\.com)(.*)"
+
 # regex path patterns
 PATH_REGEX_MAIN = r"^/restapis/([A-Za-z0-9_\-]+)/[a-z]+(\?.*)?"
 PATH_REGEX_SUB = r"^/restapis/([A-Za-z0-9_\-]+)/[a-z]+/([A-Za-z0-9_\-]+)/.*"
@@ -49,7 +65,7 @@ APIGATEWAY_SQS_DATA_INBOUND_TEMPLATE = (
 # special tag name to allow specifying a custom ID for new REST APIs
 TAG_KEY_CUSTOM_ID = "_custom_id_"
 
-# map API IDs to region names
+# map API IDs to region names - TODO remove and replace with in-memory lookup
 API_REGIONS = {}
 
 # TODO: make the CRUD operations in this file generic for the different model types (authorizes, validators, ...)
@@ -73,6 +89,8 @@ class APIGatewayRegion(RegionBackend):
     vpc_links: Dict[str, Dict]
     # maps cert ID to client certificate details
     client_certificates: Dict[str, Dict]
+    # maps resource ARN to tags
+    TAGS: Dict[str, Dict[str, str]] = {}
 
     def __init__(self):
         self.authorizers = {}
@@ -90,6 +108,78 @@ class APIGatewayRegion(RegionBackend):
         self.client_certificates = {}
 
 
+class Resolver:
+    def __init__(self, document: dict, allow_recursive=True):
+        self.document = document
+        self.allow_recursive = allow_recursive
+        # cache which maps known refs to part of the document
+        self._cache = {}
+        self._refpaths = ["#"]
+
+    def _is_ref(self, item) -> bool:
+        return isinstance(item, dict) and "$ref" in item
+
+    def _is_internal_ref(self, refpath) -> bool:
+        return str(refpath).startswith("#/")
+
+    @property
+    def current_path(self):
+        return self._refpaths[-1]
+
+    @contextlib.contextmanager
+    def _pathctx(self, refpath: str):
+        if not self._is_internal_ref(refpath):
+            refpath = "/".join((self.current_path, refpath))
+
+        self._refpaths.append(refpath)
+        yield
+        self._refpaths.pop()
+
+    def _resolve_refpath(self, refpath: str) -> dict:
+        if refpath in self._refpaths and not self.allow_recursive:
+            raise Exception("recursion detected with allow_recursive=False")
+
+        if refpath in self._cache:
+            return self._cache.get(refpath)
+
+        with self._pathctx(refpath):
+            if self._is_internal_ref(self.current_path):
+                cur = self.document
+            else:
+                raise NotImplementedError("External references not yet supported.")
+
+            for step in self.current_path.split("/")[1:]:
+                cur = cur.get(step)
+
+            self._cache[self.current_path] = cur
+            return cur
+
+    def _namespaced_resolution(self, namespace: str, data: Union[dict, list]) -> Union[dict, list]:
+        with self._pathctx(namespace):
+            return self._resolve_references(data)
+
+    def _resolve_references(self, data) -> Union[dict, list]:
+        if self._is_ref(data):
+            return self._resolve_refpath(data["$ref"])
+
+        if isinstance(data, dict):
+            for k, v in data.items():
+                data[k] = self._namespaced_resolution(k, v)
+        elif isinstance(data, list):
+            for i, v in enumerate(data):
+                data[i] = self._namespaced_resolution(str(i), v)
+
+        return data
+
+    def resolve_references(self) -> dict:
+        return self._resolve_references(self.document)
+
+
+def resolve_references(data: dict, allow_recursive=True) -> dict:
+    resolver = Resolver(data, allow_recursive=allow_recursive)
+    return resolver.resolve_references()
+
+
 def make_json_response(message):
     return requests_response(json.dumps(message), headers={"Content-Type": APPLICATION_JSON})
 
@@ -108,595 +198,11 @@ def make_accepted_response():
 
 
 def get_api_id_from_path(path):
-    match = re.match(PATH_REGEX_SUB, path)
-    if match:
+    if match := re.match(PATH_REGEX_SUB, path):
         return match.group(1)
     return re.match(PATH_REGEX_MAIN, path).group(1)
 
 
-# -------------
-# ACCOUNT APIs
-# -------------
-
-
-def get_account():
-    region_details = APIGatewayRegion.get()
-    return to_account_response_json(region_details.account)
-
-
-def update_account(data):
-    region_details = APIGatewayRegion.get()
-    apply_json_patch_safe(region_details.account, data["patchOperations"], in_place=True)
-    return to_account_response_json(region_details.account)
-
-
-def handle_accounts(method, path, data, headers):
-    if method == "GET":
-        return get_account()
-    if method == "PATCH":
-        return update_account(data)
-    return make_error_response("Not implemented for API Gateway accounts: %s" % method, code=404)
-
-
-# -----------------
-# AUTHORIZERS APIs
-# -----------------
-
-
-def get_authorizer_id_from_path(path):
-    match = re.match(PATH_REGEX_AUTHORIZERS, path)
-    return match.group(2) if match else None
-
-
-def _find_authorizer(api_id, authorizer_id):
-    return find_api_subentity_by_id(api_id, authorizer_id, "authorizers")
-
-
-def normalize_authorizer(data):
-    is_list = isinstance(data, list)
-    entries = data if is_list else [data]
-    for i in range(len(entries)):
-        entry = common.clone(entries[i])
-        # terraform sends this as a string in patch, so convert to int
-        entry["authorizerResultTtlInSeconds"] = int(entry.get("authorizerResultTtlInSeconds", 300))
-        entries[i] = entry
-    return entries if is_list else entries[0]
-
-
-def get_authorizers(path):
-    region_details = APIGatewayRegion.get()
-
-    # This function returns either a list or a single authorizer (depending on the path)
-    api_id = get_api_id_from_path(path)
-    authorizer_id = get_authorizer_id_from_path(path)
-
-    auth_list = region_details.authorizers.get(api_id) or []
-
-    if authorizer_id:
-        authorizer = _find_authorizer(api_id, authorizer_id)
-        if authorizer is None:
-            return make_error_response(
-                "Authorizer not found: %s" % authorizer_id,
-                code=404,
-                error_type="NotFoundException",
-            )
-        return to_authorizer_response_json(api_id, authorizer)
-
-    result = [to_authorizer_response_json(api_id, a) for a in auth_list]
-    result = {"item": result}
-    return result
-
-
-def add_authorizer(path, data):
-    region_details = APIGatewayRegion.get()
-
-    api_id = get_api_id_from_path(path)
-    authorizer_id = common.short_uid()[:6]  # length 6 to make TF tests pass
-    result = common.clone(data)
-
-    result["id"] = authorizer_id
-    result = normalize_authorizer(result)
-    region_details.authorizers.setdefault(api_id, []).append(result)
-
-    return make_json_response(to_authorizer_response_json(api_id, result))
-
-
-def update_authorizer(path, data):
-    region_details = APIGatewayRegion.get()
-
-    api_id = get_api_id_from_path(path)
-    authorizer_id = get_authorizer_id_from_path(path)
-
-    authorizer = _find_authorizer(api_id, authorizer_id)
-    if authorizer is None:
-        return make_error_response("Authorizer not found for API: %s" % api_id, code=404)
-
-    result = apply_json_patch_safe(authorizer, data["patchOperations"])
-    result = normalize_authorizer(result)
-
-    auth_list = region_details.authorizers[api_id]
-    for i in range(len(auth_list)):
-        if auth_list[i]["id"] == authorizer_id:
-            auth_list[i] = result
-
-    return make_json_response(to_authorizer_response_json(api_id, result))
-
-
-def delete_authorizer(path):
-    region_details = APIGatewayRegion.get()
-
-    api_id = get_api_id_from_path(path)
-    authorizer_id = get_authorizer_id_from_path(path)
-
-    auth_list = region_details.authorizers[api_id]
-    for i in range(len(auth_list)):
-        if auth_list[i]["id"] == authorizer_id:
-            del auth_list[i]
-            break
-
-    return make_accepted_response()
-
-
-def handle_authorizers(method, path, data, headers):
-    if method == "GET":
-        return get_authorizers(path)
-    if method == "POST":
-        return add_authorizer(path, data)
-    if method == "PATCH":
-        return update_authorizer(path, data)
-    if method == "DELETE":
-        return delete_authorizer(path)
-    return make_error_response("Not implemented for API Gateway authorizers: %s" % method, code=404)
-
-
-# -------------------------
-# DOCUMENTATION PARTS APIs
-# -------------------------
-
-
-def get_documentation_part_id_from_path(path):
-    match = re.match(PATH_REGEX_DOC_PARTS, path)
-    return match.group(2) if match else None
-
-
-def _find_documentation_part(api_id, documentation_part_id):
-    return find_api_subentity_by_id(api_id, documentation_part_id, "documentation_parts")
-
-
-def get_documentation_parts(path):
-    region_details = APIGatewayRegion.get()
-
-    # This function returns either a list or a single entity (depending on the path)
-    api_id = get_api_id_from_path(path)
-    entity_id = get_documentation_part_id_from_path(path)
-
-    auth_list = region_details.documentation_parts.get(api_id) or []
-
-    if entity_id:
-        entity = _find_documentation_part(api_id, entity_id)
-        if entity is None:
-            return make_error_response(
-                "Documentation part not found: %s" % entity_id,
-                code=404,
-                error_type="NotFoundException",
-            )
-        return to_documentation_part_response_json(api_id, entity)
-
-    result = [to_documentation_part_response_json(api_id, a) for a in auth_list]
-    result = {"item": result}
-    return result
-
-
-def add_documentation_part(path, data):
-    region_details = APIGatewayRegion.get()
-
-    api_id = get_api_id_from_path(path)
-    entity_id = common.short_uid()[:6]  # length 6 to make TF tests pass
-    result = common.clone(data)
-
-    result["id"] = entity_id
-    region_details.documentation_parts.setdefault(api_id, []).append(result)
-
-    return make_json_response(to_documentation_part_response_json(api_id, result))
-
-
-def update_documentation_part(path, data):
-    region_details = APIGatewayRegion.get()
-
-    api_id = get_api_id_from_path(path)
-    entity_id = get_documentation_part_id_from_path(path)
-
-    entity = _find_documentation_part(api_id, entity_id)
-    if entity is None:
-        return make_error_response("Documentation part not found for API: %s" % api_id, code=404)
-
-    result = apply_json_patch_safe(entity, data["patchOperations"])
-
-    auth_list = region_details.documentation_parts[api_id]
-    for i in range(len(auth_list)):
-        if auth_list[i]["id"] == entity_id:
-            auth_list[i] = result
-
-    return make_json_response(to_documentation_part_response_json(api_id, result))
-
-
-def delete_documentation_part(path):
-    region_details = APIGatewayRegion.get()
-
-    api_id = get_api_id_from_path(path)
-    entity_id = get_documentation_part_id_from_path(path)
-
-    auth_list = region_details.documentation_parts[api_id]
-    for i in range(len(auth_list)):
-        if auth_list[i]["id"] == entity_id:
-            del auth_list[i]
-            break
-
-    return make_accepted_response()
-
-
-def handle_documentation_parts(method, path, data, headers):
-    if method == "GET":
-        return get_documentation_parts(path)
-    if method == "POST":
-        return add_documentation_part(path, data)
-    if method == "PATCH":
-        return update_documentation_part(path, data)
-    if method == "DELETE":
-        return delete_documentation_part(path)
-    return make_error_response(
-        "Not implemented for API Gateway documentation parts: %s" % method, code=404
-    )
-
-
-# -----------------------
-# BASE PATH MAPPING APIs
-# -----------------------
-
-
-def get_domain_from_path(path):
-    matched = re.match(PATH_REGEX_PATH_MAPPINGS, path)
-    return matched.group(1) if matched else None
-
-
-def get_base_path_from_path(path):
-    return re.match(PATH_REGEX_PATH_MAPPINGS, path).group(2)
-
-
-def get_base_path_mapping(path):
-    region_details = APIGatewayRegion.get()
-
-    # This function returns either a list or a single mapping (depending on the path)
-    domain_name = get_domain_from_path(path)
-    base_path = get_base_path_from_path(path)
-
-    mappings_list = region_details.base_path_mappings.get(domain_name) or []
-
-    if base_path:
-        mapping = ([m for m in mappings_list if m["basePath"] == base_path] or [None])[0]
-        if mapping is None:
-            return make_error_response(
-                "Base path mapping not found: %s" % base_path,
-                code=404,
-                error_type="NotFoundException",
-            )
-        return to_base_mapping_response_json(domain_name, base_path, mapping)
-
-    result = [to_base_mapping_response_json(domain_name, m["basePath"], m) for m in mappings_list]
-    result = {"item": result}
-    return result
-
-
-def add_base_path_mapping(path, data):
-    region_details = APIGatewayRegion.get()
-
-    domain_name = get_domain_from_path(path)
-    # Note: "(none)" is a special value in API GW:
-    # https://docs.aws.amazon.com/apigateway/api-reference/link-relation/basepathmapping-by-base-path
-    base_path = data["basePath"] = data.get("basePath") or "(none)"
-    result = common.clone(data)
-
-    region_details.base_path_mappings.setdefault(domain_name, []).append(result)
-
-    return make_json_response(to_base_mapping_response_json(domain_name, base_path, result))
-
-
-def update_base_path_mapping(path, data):
-    region_details = APIGatewayRegion.get()
-
-    domain_name = get_domain_from_path(path)
-    base_path = get_base_path_from_path(path)
-
-    mappings_list = region_details.base_path_mappings.get(domain_name) or []
-
-    mapping = ([m for m in mappings_list if m["basePath"] == base_path] or [None])[0]
-    if mapping is None:
-        return make_error_response(
-            "Not found: mapping for domain name %s, base path %s in list %s"
-            % (domain_name, base_path, mappings_list),
-            code=404,
-        )
-
-    operations = data["patchOperations"]
-    operations = operations if isinstance(operations, list) else [operations]
-    for operation in operations:
-        if operation["path"] == "/restapiId":
-            operation["path"] = "/restApiId"
-    result = apply_json_patch_safe(mapping, operations)
-
-    for i in range(len(mappings_list)):
-        if mappings_list[i]["basePath"] == base_path:
-            mappings_list[i] = result
-
-    return make_json_response(to_base_mapping_response_json(domain_name, base_path, result))
-
-
-def delete_base_path_mapping(path):
-    region_details = APIGatewayRegion.get()
-
-    domain_name = get_domain_from_path(path)
-    base_path = get_base_path_from_path(path)
-
-    mappings_list = region_details.base_path_mappings.get(domain_name) or []
-    for i in range(len(mappings_list)):
-        if mappings_list[i]["basePath"] == base_path:
-            del mappings_list[i]
-            return make_accepted_response()
-
-    return make_error_response(
-        "Base path mapping %s for domain %s not found" % (base_path, domain_name),
-        code=404,
-    )
-
-
-def handle_base_path_mappings(method, path, data, headers):
-    path = urlparse.unquote(path)
-    if method == "GET":
-        return get_base_path_mapping(path)
-    if method == "POST":
-        return add_base_path_mapping(path, data)
-    if method == "PATCH":
-        return update_base_path_mapping(path, data)
-    if method == "DELETE":
-        return delete_base_path_mapping(path)
-    return make_error_response(
-        "Not implemented for API Gateway base path mappings: %s" % method, code=404
-    )
-
-
-# ------------------------
-# CLIENT CERTIFICATE APIs
-# ------------------------
-
-
-def get_cert_id_from_path(path):
-    matched = re.match(PATH_REGEX_CLIENT_CERTS, path)
-    return matched.group(1) if matched else None
-
-
-def get_client_certificate(path):
-    region_details = APIGatewayRegion.get()
-    cert_id = get_cert_id_from_path(path)
-    result = region_details.client_certificates.get(cert_id)
-    if result is None:
-        return make_error_response('Client certificate ID "%s" not found' % cert_id, code=404)
-    return result
-
-
-def add_client_certificate(path, data):
-    region_details = APIGatewayRegion.get()
-    result = common.clone(data)
-    result["clientCertificateId"] = cert_id = common.short_uid()
-    result["createdDate"] = common.now_utc()
-    result["expirationDate"] = result["createdDate"] + 60 * 60 * 24 * 30  # assume 30 days validity
-    result["pemEncodedCertificate"] = "testcert-123"  # TODO return proper certificate!
-    region_details.client_certificates[cert_id] = result
-    return make_json_response(to_client_cert_response_json(result))
-
-
-def update_client_certificate(path, data):
-    region_details = APIGatewayRegion.get()
-    entity_id = get_cert_id_from_path(path)
-    entity = region_details.client_certificates.get(entity_id)
-    if entity is None:
-        return make_error_response('Client certificate ID "%s" not found' % entity_id, code=404)
-    result = apply_json_patch_safe(entity, data["patchOperations"])
-    return make_json_response(to_client_cert_response_json(result))
-
-
-def delete_client_certificate(path):
-    region_details = APIGatewayRegion.get()
-    entity_id = get_cert_id_from_path(path)
-    entity = region_details.client_certificates.pop(entity_id, None)
-    if entity is None:
-        return make_error_response('VPC link ID "%s" not found for deletion' % entity_id, code=404)
-    return make_accepted_response()
-
-
-def handle_client_certificates(method, path, data, headers):
-    if method == "GET":
-        return get_client_certificate(path)
-    if method == "POST":
-        return add_client_certificate(path, data)
-    if method == "PATCH":
-        return update_client_certificate(path, data)
-    if method == "DELETE":
-        return delete_client_certificate(path)
-    return make_error_response(
-        "Not implemented for API Gateway base path mappings: %s" % method, code=404
-    )
-
-
-# --------------
-# VCP LINK APIs
-# --------------
-
-
-def get_vpc_links(path):
-    region_details = APIGatewayRegion.get()
-    vpc_link_id = get_vpc_link_id_from_path(path)
-    if vpc_link_id:
-        vpc_link = region_details.vpc_links.get(vpc_link_id)
-        if vpc_link is None:
-            return make_error_response('VPC link ID "%s" not found' % vpc_link_id, code=404)
-        return make_json_response(to_vpc_link_response_json(vpc_link))
-    result = region_details.vpc_links.values()
-    result = [to_vpc_link_response_json(r) for r in result]
-    result = {"items": result}
-    return result
-
-
-def add_vpc_link(path, data):
-    region_details = APIGatewayRegion.get()
-    result = common.clone(data)
-    result["id"] = common.short_uid()
-    result["status"] = "AVAILABLE"
-    region_details.vpc_links[result["id"]] = result
-    return make_json_response(to_vpc_link_response_json(result))
-
-
-def update_vpc_link(path, data):
-    region_details = APIGatewayRegion.get()
-    vpc_link_id = get_vpc_link_id_from_path(path)
-    vpc_link = region_details.vpc_links.get(vpc_link_id)
-    if vpc_link is None:
-        return make_error_response('VPC link ID "%s" not found' % vpc_link_id, code=404)
-    result = apply_json_patch_safe(vpc_link, data["patchOperations"])
-    return make_json_response(to_vpc_link_response_json(result))
-
-
-def delete_vpc_link(path):
-    region_details = APIGatewayRegion.get()
-    vpc_link_id = get_vpc_link_id_from_path(path)
-    vpc_link = region_details.vpc_links.pop(vpc_link_id, None)
-    if vpc_link is None:
-        return make_error_response(
-            'VPC link ID "%s" not found for deletion' % vpc_link_id, code=404
-        )
-    return make_accepted_response()
-
-
-def get_vpc_link_id_from_path(path):
-    match = re.match(PATH_REGEX_VPC_LINKS, path)
-    return match.group(1) if match else None
-
-
-def handle_vpc_links(method, path, data, headers):
-    if method == "GET":
-        return get_vpc_links(path)
-    if method == "POST":
-        return add_vpc_link(path, data)
-    if method == "PATCH":
-        return update_vpc_link(path, data)
-    if method == "DELETE":
-        return delete_vpc_link(path)
-    return make_error_response("Not implemented for API Gateway VPC links: %s" % method, code=404)
-
-
-# ----------------
-# VALIDATORS APIs
-# ----------------
-
-
-def get_validator_id_from_path(path):
-    match = re.match(PATH_REGEX_VALIDATORS, path)
-    return match.group(2) if match else None
-
-
-def _find_validator(api_id, validator_id):
-    region_details = APIGatewayRegion.get()
-    auth_list = region_details.validators.get(api_id) or []
-    return ([a for a in auth_list if a["id"] == validator_id] or [None])[0]
-
-
-def get_validators(path):
-    region_details = APIGatewayRegion.get()
-
-    # This function returns either a list or a single validator (depending on the path)
-    api_id = get_api_id_from_path(path)
-    validator_id = get_validator_id_from_path(path)
-
-    auth_list = region_details.validators.get(api_id) or []
-
-    if validator_id:
-        validator = _find_validator(api_id, validator_id)
-        if validator is None:
-            return make_error_response(
-                "Validator %s for API Gateway %s not found" % (validator_id, api_id),
-                code=404,
-            )
-        return to_validator_response_json(api_id, validator)
-
-    result = [to_validator_response_json(api_id, a) for a in auth_list]
-    result = {"item": result}
-    return result
-
-
-def add_validator(path, data):
-    region_details = APIGatewayRegion.get()
-
-    api_id = get_api_id_from_path(path)
-    validator_id = common.short_uid()[:6]  # length 6 (as in AWS) to make TF tests pass
-    result = common.clone(data)
-    result["id"] = validator_id
-
-    region_details.validators.setdefault(api_id, []).append(result)
-
-    return result
-
-
-def update_validator(path, data):
-    region_details = APIGatewayRegion.get()
-
-    api_id = get_api_id_from_path(path)
-    validator_id = get_validator_id_from_path(path)
-
-    validator = _find_validator(api_id, validator_id)
-    if validator is None:
-        return make_error_response(
-            "Validator %s for API Gateway %s not found" % (validator_id, api_id),
-            code=404,
-        )
-
-    result = apply_json_patch_safe(validator, data["patchOperations"])
-
-    entry_list = region_details.validators[api_id]
-    for i in range(len(entry_list)):
-        if entry_list[i]["id"] == validator_id:
-            entry_list[i] = result
-
-    return make_json_response(to_validator_response_json(api_id, result))
-
-
-def delete_validator(path):
-    region_details = APIGatewayRegion.get()
-
-    api_id = get_api_id_from_path(path)
-    validator_id = get_validator_id_from_path(path)
-
-    auth_list = region_details.validators[api_id]
-    for i in range(len(auth_list)):
-        if auth_list[i]["id"] == validator_id:
-            del auth_list[i]
-            return make_accepted_response()
-
-    return make_error_response(
-        "Validator %s for API Gateway %s not found" % (validator_id, api_id), code=404
-    )
-
-
-def handle_validators(method, path, data, headers):
-    if method == "GET":
-        return get_validators(path)
-    if method == "POST":
-        return add_validator(path, data)
-    if method == "PATCH":
-        return update_validator(path, data)
-    if method == "DELETE":
-        return delete_validator(path)
-    return make_error_response("Not implemented for API Gateway validators: %s" % method, code=404)
-
-
 def is_test_invoke_method(method, path):
     return method == "POST" and bool(re.match(PATH_REGEX_TEST_INVOKE_API, path))
 
@@ -840,56 +346,6 @@ def handle_gateway_responses(method, path, data, headers):
 # ---------------
 
 
-def to_authorizer_response_json(api_id, data):
-    return to_response_json("authorizer", data, api_id=api_id)
-
-
-def to_validator_response_json(api_id, data):
-    return to_response_json("validator", data, api_id=api_id)
-
-
-def to_documentation_part_response_json(api_id, data):
-    return to_response_json("documentationpart", data, api_id=api_id)
-
-
-def to_base_mapping_response_json(domain_name, base_path, data):
-    self_link = "/domainnames/%s/basepathmappings/%s" % (domain_name, base_path)
-    return to_response_json("basepathmapping", data, self_link=self_link)
-
-
-def to_account_response_json(data):
-    return to_response_json("account", data, self_link="/account")
-
-
-def to_vpc_link_response_json(data):
-    return to_response_json("vpclink", data)
-
-
-def to_client_cert_response_json(data):
-    return to_response_json("clientcertificate", data, id_attr="clientCertificateId")
-
-
-def to_response_json(model_type, data, api_id=None, self_link=None, id_attr=None):
-    if isinstance(data, list) and len(data) == 1:
-        data = data[0]
-    id_attr = id_attr or "id"
-    result = common.clone(data)
-    if not self_link:
-        self_link = "/%ss/%s" % (model_type, data[id_attr])
-        if api_id:
-            self_link = "/restapis/%s/%s" % (api_id, self_link)
-    if "_links" not in result:
-        result["_links"] = {}
-    result["_links"]["self"] = {"href": self_link}
-    result["_links"]["curies"] = {
-        "href": "https://docs.aws.amazon.com/apigateway/latest/developerguide/restapi-authorizer-latest.html",
-        "name": model_type,
-        "templated": True,
-    }
-    result["_links"]["%s:delete" % model_type] = {"href": self_link}
-    return result
-
-
 def find_api_subentity_by_id(api_id, entity_id, map_name):
     region_details = APIGatewayRegion.get()
     auth_list = getattr(region_details, map_name).get(api_id) or []
@@ -1135,10 +591,16 @@ def apply_json_patch_safe(subject, patch_operations, in_place=True, return_list=
 
 
 def import_api_from_openapi_spec(
-    rest_api: apigateway_models.RestAPI, function_id: str, body: Dict, query_params: Dict
+    rest_api: apigateway_models.RestAPI, body: Dict, query_params: Dict
 ) -> apigateway_models.RestAPI:
     """Import an API from an OpenAPI spec document"""
 
+    resolved_schema = resolve_references(body)
+    # XXX for some reason this makes cf tests fail that's why is commented.
+    # test_cfn_handle_serverless_api_resource
+    # rest_api.name = resolved_schema.get("info", {}).get("title")
+    rest_api.description = resolved_schema.get("info", {}).get("description")
+
     # Remove default root, then add paths from API spec
     rest_api.resources = {}
 
@@ -1149,12 +611,11 @@ def import_api_from_openapi_spec(
             parent_path = "/".join(parts[:-1])
             parent = get_or_create_path(parent_path)
             parent_id = parent.id
-        existing = [
+        if existing := [
             r
             for r in rest_api.resources.values()
             if r.path_part == (parts[-1] or "/") and (r.parent_id or "") == (parent_id or "")
-        ]
-        if existing:
+        ]:
             return existing[0]
         return add_path(path, parts, parent_id=parent_id)
 
@@ -1168,45 +629,214 @@ def import_api_from_openapi_spec(
             path_part=parts[-1] or "/",
             parent_id=parent_id,
         )
-        for m, payload in body["paths"].get(path, {}).items():
-            m = m.upper()
-            payload = payload["x-amazon-apigateway-integration"]
 
-            child.add_method(m, None, None)
+        for method, method_schema in resolved_schema["paths"].get(path, {}).items():
+            method = method.upper()
+            method_resource = child.add_method(method, None, None)
+            method_integration = method_schema.get("x-amazon-apigateway-integration", {})
+            responses = method_schema.get("responses", {})
+            for status_code in responses:
+                response_model = None
+                if model_schema := responses.get(status_code, {}).get("schema", {}):
+                    response_model = {APPLICATION_JSON: model_schema}
+
+                response_parameters = (
+                    method_integration.get("responses", {})
+                    .get("default", {})
+                    .get("responseParameters")
+                )
+                method_resource.create_response(
+                    status_code,
+                    response_model,
+                    response_parameters,
+                )
+
             integration = apigateway_models.Integration(
-                http_method=m,
-                uri=payload.get("uri"),
-                integration_type=payload["type"],
-                pass_through_behavior=payload.get("passthroughBehavior"),
-                request_templates=payload.get("requestTemplates") or {},
+                http_method=method,
+                uri=method_integration.get("uri"),
+                integration_type=method_integration["type"],
+                passthrough_behavior=method_integration.get("passthroughBehavior"),
+                request_templates=method_integration.get("requestTemplates") or {},
             )
             integration.create_integration_response(
-                status_code=payload.get("responses", {}).get("default", {}).get("statusCode", 200),
+                status_code=method_integration.get("default", {}).get("statusCode", 200),
                 selection_pattern=None,
-                response_templates=None,
+                response_templates=method_integration.get("default", {}).get(
+                    "responseTemplates", None
+                ),
                 content_handling=None,
             )
-            child.resource_methods[m]["methodIntegration"] = integration
+            child.resource_methods[method]["methodIntegration"] = integration
 
         rest_api.resources[child_id] = child
         return child
 
+    if definitions := resolved_schema.get("definitions", {}):
+        for name, model in definitions.items():
+            rest_api.add_model(name=name, schema=model, content_type=APPLICATION_JSON)
+
     basepath_mode = (query_params.get("basepath") or ["prepend"])[0]
-    base_path = (body.get("basePath") or "") if basepath_mode == "prepend" else ""
-    for path in body.get("paths", {}):
+    base_path = (resolved_schema.get("basePath") or "") if basepath_mode == "prepend" else ""
+    for path in resolved_schema.get("paths", {}):
         get_or_create_path(base_path + path)
 
-    policy = body.get("x-amazon-apigateway-policy")
+    policy = resolved_schema.get("x-amazon-apigateway-policy")
     if policy:
         policy = json.dumps(policy) if isinstance(policy, dict) else str(policy)
         rest_api.policy = policy
-    minimum_compression_size = body.get("x-amazon-apigateway-minimum-compression-size")
+    minimum_compression_size = resolved_schema.get("x-amazon-apigateway-minimum-compression-size")
     if minimum_compression_size is not None:
         rest_api.minimum_compression_size = int(minimum_compression_size)
-    endpoint_config = body.get("x-amazon-apigateway-endpoint-configuration")
+    endpoint_config = resolved_schema.get("x-amazon-apigateway-endpoint-configuration")
     if endpoint_config:
         if endpoint_config.get("vpcEndpointIds"):
             endpoint_config.setdefault("types", ["PRIVATE"])
         rest_api.endpoint_configuration = endpoint_config
 
     return rest_api
+
+
+def get_target_resource_details(invocation_context: ApiInvocationContext) -> Tuple[str, Dict]:
+    """Look up and return the API GW resource (path pattern + resource dict) for the given invocation context."""
+    path_map = get_rest_api_paths(
+        rest_api_id=invocation_context.api_id, region_name=invocation_context.region_name
+    )
+    relative_path = invocation_context.invocation_path
+    try:
+        extracted_path, resource = get_resource_for_path(path=relative_path, path_map=path_map)
+        invocation_context.resource = resource
+        return extracted_path, resource
+    except Exception:
+        return None, None
+
+
+def get_target_resource_method(invocation_context: ApiInvocationContext) -> Optional[Dict]:
+    """Look up and return the API GW resource method for the given invocation context."""
+    _, resource = get_target_resource_details(invocation_context)
+    if not resource:
+        return None
+    methods = resource.get("resourceMethods") or {}
+    method_name = invocation_context.method.upper()
+    return methods.get(method_name) or methods.get("ANY")
+
+
+def get_event_request_context(invocation_context: ApiInvocationContext):
+    method = invocation_context.method
+    path = invocation_context.path
+    headers = invocation_context.headers
+    integration_uri = invocation_context.integration_uri
+    resource_path = invocation_context.resource_path
+    resource_id = invocation_context.resource_id
+
+    set_api_id_stage_invocation_path(invocation_context)
+    relative_path, query_string_params = extract_query_string_params(
+        path=invocation_context.path_with_query_string
+    )
+    api_id = invocation_context.api_id
+    stage = invocation_context.stage
+
+    source_ip = headers.get("X-Forwarded-For", ",").split(",")[-2].strip()
+    integration_uri = integration_uri or ""
+    account_id = integration_uri.split(":lambda:path")[-1].split(":function:")[0].split(":")[-1]
+    account_id = account_id or TEST_AWS_ACCOUNT_ID
+    request_context = {
+        "accountId": account_id,
+        "apiId": api_id,
+        "resourcePath": resource_path or relative_path,
+        "domainPrefix": invocation_context.domain_prefix,
+        "domainName": invocation_context.domain_name,
+        "resourceId": resource_id,
+        "requestId": long_uid(),
+        "identity": {
+            "accountId": account_id,
+            "sourceIp": source_ip,
+            "userAgent": headers.get("User-Agent"),
+        },
+        "httpMethod": method,
+        "protocol": "HTTP/1.1",
+        "requestTime": pytz.utc.localize(datetime.datetime.utcnow()).strftime(
+            REQUEST_TIME_DATE_FORMAT
+        ),
+        "requestTimeEpoch": int(time.time() * 1000),
+        "authorizer": {},
+    }
+
+    # set "authorizer" and "identity" event attributes from request context
+    auth_context = invocation_context.auth_context
+    if auth_context:
+        request_context["authorizer"] = auth_context
+    request_context["identity"].update(invocation_context.auth_identity or {})
+
+    if not is_test_invoke_method(method, path):
+        request_context["path"] = (f"/{stage}" if stage else "") + relative_path
+        request_context["stage"] = stage
+    return request_context
+
+
+def set_api_id_stage_invocation_path(
+    invocation_context: ApiInvocationContext,
+) -> ApiInvocationContext:
+    # skip if all details are already available
+    values = (
+        invocation_context.api_id,
+        invocation_context.stage,
+        invocation_context.path_with_query_string,
+    )
+    if all(values):
+        return invocation_context
+
+    # skip if this is a websocket request
+    if invocation_context.is_websocket_request():
+        return invocation_context
+
+    path = invocation_context.path
+    headers = invocation_context.headers
+
+    path_match = re.search(PATH_REGEX_USER_REQUEST, path)
+    host_header = headers.get(HEADER_LOCALSTACK_EDGE_URL, "") or headers.get("Host") or ""
+    host_match = re.search(HOST_REGEX_EXECUTE_API, host_header)
+    test_invoke_match = re.search(PATH_REGEX_TEST_INVOKE_API, path)
+    if path_match:
+        api_id = path_match.group(1)
+        stage = path_match.group(2)
+        relative_path_w_query_params = "/%s" % path_match.group(3)
+    elif host_match:
+        api_id = extract_api_id_from_hostname_in_url(host_header)
+        stage = path.strip("/").split("/")[0]
+        relative_path_w_query_params = "/%s" % path.lstrip("/").partition("/")[2]
+    elif test_invoke_match:
+        # special case: fetch the resource details for TestInvokeApi invocations
+        stage = None
+        region_name = invocation_context.region_name
+        api_id = test_invoke_match.group(1)
+        resource_id = test_invoke_match.group(2)
+        query_string = test_invoke_match.group(4) or ""
+        apigateway = aws_stack.connect_to_service(
+            service_name="apigateway", region_name=region_name
+        )
+        resource = apigateway.get_resource(restApiId=api_id, resourceId=resource_id)
+        resource_path = resource.get("path")
+        relative_path_w_query_params = f"{resource_path}{query_string}"
+    else:
+        raise Exception(
+            f"Unable to extract API Gateway details from request: {path} {dict(headers)}"
+        )
+    if api_id:
+        # set current region in request thread local, to ensure aws_stack.get_region() works properly
+        # TODO: replace with RequestContextManager
+        if getattr(THREAD_LOCAL, "request_context", None) is not None:
+            api_region = API_REGIONS.get(api_id, "")
+            THREAD_LOCAL.request_context.headers[MARKER_APIGW_REQUEST_REGION] = api_region
+
+    # set details in invocation context
+    invocation_context.api_id = api_id
+    invocation_context.stage = stage
+    invocation_context.path_with_query_string = relative_path_w_query_params
+    return invocation_context
+
+
+def extract_api_id_from_hostname_in_url(hostname: str) -> str:
+    """Extract API ID 'id123' from URLs like https://id123.execute-api.localhost.localstack.cloud:4566"""
+    match = re.match(HOST_REGEX_EXECUTE_API, hostname)
+    api_id = match.group(1)
+    return api_id
