@@ -25,6 +25,14 @@ class FirehoseDeliveryStream(GenericBaseModel):
                     "DeliveryStreamType",
                     "S3DestinationConfiguration",
                     "ElasticsearchDestinationConfiguration",
+                    "AmazonopensearchserviceDestinationConfiguration",
+                    "DeliveryStreamEncryptionConfigurationInput",
+                    "ExtendedS3DestinationConfiguration",
+                    "HttpEndpointDestinationConfiguration",
+                    "KinesisStreamSourceConfiguration",
+                    "RedshiftDestinationConfiguration",
+                    "SplunkDestinationConfiguration",
+                    "Tags",
                 ),
             },
             "delete": {
