@@ -1,36 +1,30 @@
 import json
+import logging
 import re
-from typing import Dict
 from urllib.parse import urlencode
 
 import xmltodict
 from moto.sqs.models import TRANSPORT_TYPE_ENCODINGS, Message
 from moto.sqs.utils import parse_message_attributes
-from requests.models import Request, Response
+from requests.models import Request
 
 from localstack import config, constants
 from localstack.config import SQS_PORT_EXTERNAL
-from localstack.services.awslambda.lambda_api import EventSourceListener
 from localstack.services.install import SQS_BACKEND_IMPL
 from localstack.services.sns import sns_listener
+from localstack.services.sqs.sqs_utils import is_sqs_queue_url
 from localstack.utils.analytics import event_publisher
 from localstack.utils.aws import aws_stack
 from localstack.utils.aws.aws_responses import (
     calculate_crc32,
     make_requests_error,
-    parse_urlencoded_data,
     requests_response,
 )
-from localstack.utils.common import (
-    clone,
-    ensure_list,
-    get_service_protocol,
-    parse_request_data,
-    path_from_url,
-    to_str,
-)
+from localstack.utils.common import clone, get_service_protocol, parse_request_data, to_str
 from localstack.utils.persistence import PersistingProxyListener
 
+LOG = logging.getLogger(__name__)
+
 API_VERSION = "2012-11-05"
 XMLNS_SQS = "http://queue.amazonaws.com/doc/%s/" % API_VERSION
 
@@ -205,39 +199,6 @@ def _list_dead_letter_source_queues(queues, queue_url):
     return format_list_dl_source_queues_response(dead_letter_source_queues)
 
 
-def _process_sent_message(path: str, req_data: Dict[str, str], headers: Dict, response: Response):
-    """Extract one or multiple messages sent via SendMessage/SendMessageBatch from the given
-    request/response data and forward them to the Lambda EventSourceListener for further processing"""
-
-    queue_url = _queue_url(path, req_data, headers)
-    action = req_data.get("Action")
-
-    # extract data from XML response - assume data is wrapped in 2 parent elements
-    response_data = xmltodict.parse(response.content)
-
-    messages = []
-    if action == "SendMessage":
-        response_data = response_data["SendMessageResponse"]["SendMessageResult"]
-        message = clone(req_data)
-        message.update(response_data)
-        messages.append(message)
-    elif action == "SendMessageBatch":
-        response_data = response_data["SendMessageBatchResponse"]["SendMessageBatchResult"]
-        messages = parse_urlencoded_data(req_data, "SendMessageBatchRequestEntry")
-        # Note: only forwarding messages from 'Successful', not from 'Failed' list
-        entries = response_data.get("SendMessageBatchResultEntry") or []
-        entries = ensure_list(entries)
-        for successful in entries:
-            msg = [m for m in messages if m["Id"] == successful["Id"]][0]
-            msg.update(successful)
-
-    event = {
-        "QueueUrl": queue_url,
-        "Messages": messages,
-    }
-    EventSourceListener.process_event_via_listener("sqs", event)
-
-
 def format_list_dl_source_queues_response(queues):
     content_str = """<ListDeadLetterSourceQueuesResponse xmlns="{}">
                         <ListDeadLetterSourceQueuesResult>
@@ -276,11 +237,6 @@ def validate_empty_message_batch(data, req_data):
     return False
 
 
-def is_sqs_queue_url(url):
-    path = path_from_url(url).partition("?")[0]
-    return re.match(r"^/(queue|%s)/[a-zA-Z0-9_-]+$" % constants.TEST_AWS_ACCOUNT_ID, path)
-
-
 class ProxyListenerSQS(PersistingProxyListener):
     def api_name(self):
         return "sqs"
@@ -340,6 +296,9 @@ class ProxyListenerSQS(PersistingProxyListener):
                     return False
 
                 if req_data.get("QueueName").endswith(".fifo") and not _is_fifo():
+                    LOG.warn(
+                        'You are trying to create a queue ending in ".fifo".  Please use the --attributes parameter to set FifoQueue appropriately.'
+                    )
                     msg = "Can only include alphanumeric characters, hyphens, or underscores. 1 to 80 in length"
                     return make_requests_error(
                         code=400, code_string="InvalidParameterValue", message=msg
@@ -452,10 +411,6 @@ class ProxyListenerSQS(PersistingProxyListener):
                 msg = "There should be at least one SendMessageBatchRequestEntry in the request."
                 return make_requests_error(code=404, code_string="EmptyBatchRequest", message=msg)
 
-        # instruct listeners to process new SQS message
-        if action in ("SendMessage", "SendMessageBatch"):
-            _process_sent_message(path, req_data, headers, response)
-
         if content_str_original != content_str:
             # if changes have been made, return patched response
             response.headers["Content-Length"] = len(content_str)
