@@ -89,6 +89,13 @@ def get_valid_regions():
     return valid_regions
 
 
+def get_valid_regions_for_service(service_name):
+    regions = list(boto3.Session().get_available_regions(service_name))
+    regions.extend(boto3.Session().get_available_regions("cloudwatch", partition_name="aws-us-gov"))
+    regions.extend(boto3.Session().get_available_regions("cloudwatch", partition_name="aws-cn"))
+    return regions
+
+
 class Environment:
     def __init__(self, region=None, prefix=None):
         # target is the runtime environment to use, e.g.,
@@ -571,6 +578,13 @@ def extract_service_from_arn(arn: str) -> Optional[str]:
         return None
 
 
+def extract_resource_from_arn(arn: str) -> Optional[str]:
+    try:
+        return parse_arn(arn).get("resource")
+    except InvalidArnException:
+        return None
+
+
 def get_account_id(account_id=None, env=None):
     if account_id:
         return account_id
@@ -1019,10 +1033,12 @@ def create_api_gateway(
     usage_plan_name=None,
     region_name=None,
     auth_creator_func=None,  # function that receives an api_id and returns an authorizer_id
+    client=None,
 ):
     if enabled_api_keys is None:
         enabled_api_keys = []
-    client = connect_to_service("apigateway", env=env, region_name=region_name)
+    if not client:
+        client = connect_to_service("apigateway", env=env, region_name=region_name)
     resources = resources or []
     stage_name = stage_name or "testing"
     usage_plan_name = usage_plan_name or "Basic Usage"
@@ -1069,6 +1085,7 @@ def create_api_gateway(
                 integrations,
                 env=env,
                 region_name=region_name,
+                client=client,
             )
     # deploy the API gateway
     client.create_deployment(restApiId=api_id, stageName=stage_name)
@@ -1076,11 +1093,12 @@ def create_api_gateway(
 
 
 def create_api_gateway_integrations(
-    api_id, resource_id, method, integrations=None, env=None, region_name=None
+    api_id, resource_id, method, integrations=None, env=None, region_name=None, client=None
 ):
     if integrations is None:
         integrations = []
-    client = connect_to_service("apigateway", env=env, region_name=region_name)
+    if not client:
+        client = connect_to_service("apigateway", env=env, region_name=region_name)
     for integration in integrations:
         req_templates = integration.get("requestTemplates") or {}
         res_templates = integration.get("responseTemplates") or {}
