@@ -84,7 +84,7 @@ RESOURCE_TO_FUNCTION = {}
 
 
 def find_stack(stack_name):
-    from localstack.services.cloudformation.cloudformation_api import find_stack as api_find_stack
+    from localstack.services.cloudformation.provider import find_stack as api_find_stack
 
     return api_find_stack(stack_name)
 
@@ -712,6 +712,8 @@ def resolve_placeholders_in_string(result, stack):
                 )
             # make sure we resolve any functions/placeholders in the extracted string
             result = resolve_refs_recursively(stack, result)
+            # make sure we convert the result to string
+            result = "" if result is None else str(result)
             return result
         # TODO raise exception here?
         return match.group(0)
@@ -807,8 +809,8 @@ def convert_data_types(func_details, params):
             if isinstance(_obj, bool):
                 return str(_obj).lower()
             return str(_obj)
-        if _type == int:
-            return int(_obj)
+        if _type in (int, float):
+            return _type(_obj)
         return _obj
 
     def fix_types(o, **kwargs):
@@ -901,25 +903,6 @@ def configure_resource_via_sdk(stack, resource_id, resource_type, func_details,
         if action_name == "create":
             func_details["boto_client"] = "resource"
 
-    # TODO: probably move to correct cloudformation.model
-    if resource_type == "Cognito::IdentityPoolRoleAttachment":
-        if action_name == "create":
-            if func_details["function"] == "set_identity_pool_roles":
-
-                def _fix_param_mapping(params, **kwargs):
-                    role_mapping = params.get("RoleMappings")
-                    if role_mapping:
-                        fixed_mappings = {}
-                        for key, value in role_mapping.items():
-                            if value.get("IdentityProvider"):
-                                fixed_mappings[value.pop("IdentityProvider")] = value
-                            else:
-                                fixed_mappings[key] = value
-                        params["RoleMappings"] = fixed_mappings
-                    return params
-
-                func_details["parameters"] = _fix_param_mapping
-
     client = get_client(resource, func_details)
     function = getattr(client, func_details["function"])
     params = func_details.get("parameters") or lambda_get_params()
@@ -1198,7 +1181,7 @@ def add_default_resource_props(
 # -----------------------
 
 
-class TemplateDeployer(object):
+class TemplateDeployer:
     def __init__(self, stack):
         self.stack = stack
 
@@ -1471,7 +1454,7 @@ class TemplateDeployer(object):
         change_set_id=None,
         append_to_changeset=False,
     ):
-        from localstack.services.cloudformation.cloudformation_api import StackChangeSet
+        from localstack.services.cloudformation.provider import StackChangeSet
 
         old_resources = existing_stack.template["Resources"]
         new_resources = new_stack.template["Resources"]
@@ -1539,7 +1522,7 @@ class TemplateDeployer(object):
         )
 
     def apply_changes_in_loop(self, changes, stack, stack_name, action=None, new_stack=None):
-        from localstack.services.cloudformation.cloudformation_api import StackChangeSet
+        from localstack.services.cloudformation.provider import StackChangeSet
 
         def _run(*args):
             try:
