@@ -90,7 +90,7 @@ class CancellableStream(Protocol):
         raise NotImplementedError
 
 
-class PortMappings(object):
+class PortMappings:
     """Maps source to target port ranges for Docker port mappings."""
 
     def __init__(self, bind_host=None):
@@ -305,6 +305,7 @@ class ContainerConfiguration:
     user: Optional[str] = None
     cap_add: Optional[List[str]] = None
     cap_drop: Optional[List[str]] = None
+    security_opt: Optional[List[str]] = None
     network: Optional[str] = None
     dns: Optional[str] = None
     workdir: Optional[str] = None
@@ -420,12 +421,12 @@ class ContainerClient(metaclass=ABCMeta):
 
     @abstractmethod
     def pull_image(self, docker_image: str) -> None:
-        """Pulls a image with a given name from a Docker registry"""
+        """Pulls an image with a given name from a Docker registry"""
         pass
 
     @abstractmethod
     def push_image(self, docker_image: str) -> None:
-        """Pushes a image with a given name to a Docker registry"""
+        """Pushes an image with a given name to a Docker registry"""
         pass
 
     @abstractmethod
@@ -591,6 +592,8 @@ class ContainerClient(metaclass=ABCMeta):
             env_vars=container_config.env_vars,
             user=container_config.user,
             cap_add=container_config.cap_add,
+            cap_drop=container_config.cap_drop,
+            security_opt=container_config.security_opt,
             network=container_config.network,
             dns=container_config.dns,
             additional_flags=container_config.additional_flags,
@@ -615,6 +618,7 @@ class ContainerClient(metaclass=ABCMeta):
         user: Optional[str] = None,
         cap_add: Optional[List[str]] = None,
         cap_drop: Optional[List[str]] = None,
+        security_opt: Optional[List[str]] = None,
         network: Optional[str] = None,
         dns: Optional[str] = None,
         additional_flags: Optional[str] = None,
@@ -645,6 +649,7 @@ class ContainerClient(metaclass=ABCMeta):
         user: Optional[str] = None,
         cap_add: Optional[List[str]] = None,
         cap_drop: Optional[List[str]] = None,
+        security_opt: Optional[List[str]] = None,
         network: Optional[str] = None,
         dns: Optional[str] = None,
         additional_flags: Optional[str] = None,
